@@ -11,13 +11,14 @@ from threading import Thread
 from collections import OrderedDict
 from Queue import Empty
 from io import BytesIO
+from future_builtins import map
 
 from PyQt5.Qt import QObject, Qt, pyqtSignal
 
 from calibre import prints, as_unicode
-from calibre.constants import DEBUG
-from calibre.customize.ui import run_plugins_on_postimport
-from calibre.db.adding import find_books_in_directory
+from calibre.constants import DEBUG, iswindows, isosx, filesystem_encoding
+from calibre.customize.ui import run_plugins_on_postimport, run_plugins_on_postadd
+from calibre.db.adding import find_books_in_directory, compile_rule
 from calibre.db.utils import find_identical_books
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
@@ -29,6 +30,7 @@ from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs
 from calibre.utils.ipc.pool import Pool, Failure
 
+
 def validate_source(source, parent=None):  # {{{
     if isinstance(source, basestring):
         if not os.path.exists(source):
@@ -53,6 +55,7 @@ def validate_source(source, parent=None):  # {{{
     return True
 # }}}
 
+
 class Adder(QObject):
 
     do_one_signal = pyqtSignal()
@@ -119,12 +122,34 @@ class Adder(QObject):
 
     def scan(self):
 
-        def find_files(root):
-            for dirpath, dirnames, filenames in os.walk(root):
-                for files in find_books_in_directory(dirpath, self.single_book_per_directory):
-                    if self.abort_scan:
-                        return
-                    self.file_groups[len(self.file_groups)] = files
+        try:
+            compiled_rules = tuple(map(compile_rule, gprefs.get('add_filter_rules', ())))
+        except Exception:
+            compiled_rules = ()
+            import traceback
+            traceback.print_exc()
+
+        if iswindows or isosx:
+            def find_files(root):
+                for dirpath, dirnames, filenames in os.walk(root):
+                    for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
+                        if self.abort_scan:
+                            return
+                        self.file_groups[len(self.file_groups)] = files
+        else:
+            def find_files(root):
+                if isinstance(root, type(u'')):
+                    root = root.encode(filesystem_encoding)
+                for dirpath, dirnames, filenames in os.walk(root):
+                    try:
+                        dirpath = dirpath.decode(filesystem_encoding)
+                    except UnicodeDecodeError:
+                        prints('Ignoring non-decodable directory:', dirpath)
+                        continue
+                    for files in find_books_in_directory(dirpath, self.single_book_per_directory, compiled_rules=compiled_rules):
+                        if self.abort_scan:
+                            return
+                        self.file_groups[len(self.file_groups)] = files
 
         def extract(source):
             tdir = tempfile.mkdtemp(suffix='_archive', dir=self.tdir)
@@ -161,8 +186,9 @@ class Adder(QObject):
                         unreadable_files.append(path)
                 if unreadable_files:
                     if not self.file_groups:
-                        self.scan_error = _('You do not have permission to read the selected file(s).') + '\n'
-                        self.scan_error += '\n'.join(unreadable_files)
+                        m = ngettext('You do not have permission to read the selected file.',
+                                     'You do not have permission to read the selected files.', len(unreadable_files))
+                        self.scan_error = m + '\n' + '\n'.join(unreadable_files)
                     else:
                         a = self.report.append
                         for f in unreadable_files:
@@ -184,7 +210,7 @@ class Adder(QObject):
             return
         if not self.file_groups:
             error_dialog(self.pd, _('Could not add'), _(
-                'No ebook files were found in %s') % self.source, show=True)
+                'No e-book files were found in %s') % self.source, show=True)
             self.break_cycles()
             return
         self.pd.max = len(self.file_groups)
@@ -271,7 +297,8 @@ class Adder(QObject):
         a = self.report.append
         paths = self.file_groups[group_id]
         a(''), a('-' * 70)
-        a(_('Failed to read metadata from the file(s):'))
+        m = ngettext('Failed to read metadata from the file:', 'Failed to read metadata from the files:', len(paths))
+        a(m)
         [a('\t' + f) for f in paths]
         a(_('With error:')), a(details)
         mi = Metadata(_('Unknown'))
@@ -375,7 +402,7 @@ class Adder(QObject):
             [a('\t' + f) for f in paths]
             a(_('With error:')), a(traceback.format_exc())
             return
-        self.add_formats(book_id, paths, mi)
+        self.add_formats(book_id, paths, mi, is_an_add=True)
         try:
             if self.add_formats_to_existing:
                 self.db.update_data_for_find_identical_books(book_id, self.find_identical_books_data)
@@ -388,8 +415,9 @@ class Adder(QObject):
         if DEBUG:
             prints('Added', mi.title, 'to db in: %.1f' % (time.time() - st))
 
-    def add_formats(self, book_id, paths, mi, replace=True):
+    def add_formats(self, book_id, paths, mi, replace=True, is_an_add=False):
         fmap = {p.rpartition(os.path.extsep)[-1].lower():p for p in paths}
+        fmt_map = {}
         for fmt, path in fmap.iteritems():
             # The onimport plugins have already been run by the read metadata
             # worker
@@ -398,11 +426,14 @@ class Adder(QObject):
             try:
                 if self.db.add_format(book_id, fmt, path, run_hooks=False, replace=replace):
                     run_plugins_on_postimport(self.dbref(), book_id, fmt)
+                    fmt_map[fmt.lower()] = path
             except Exception:
                 a = self.report.append
                 a(''), a('-' * 70)
                 a(_('Failed to add the file {0} to the book: {1}').format(path, mi.title))
                 a(_('With error:')), a(traceback.format_exc())
+        if is_an_add:
+            run_plugins_on_postadd(self.dbref(), book_id, fmt_map)
 
     def process_duplicates(self):
         if self.duplicates:
