@@ -12,7 +12,7 @@ from functools import partial
 from PyQt5.Qt import (QPixmap, QSize, QWidget, Qt, pyqtSignal, QUrl, QIcon,
     QPropertyAnimation, QEasingCurve, QApplication, QFontInfo, QAction,
     QSizePolicy, QPainter, QRect, pyqtProperty, QLayout, QPalette, QMenu,
-    QPen, QColor)
+    QPen, QColor, QMimeData)
 from PyQt5.QtWebKitWidgets import QWebView
 
 from calibre import fit_image
@@ -27,6 +27,8 @@ from calibre.utils.img import image_from_x, blend_image
 from calibre.utils.localization import is_rtl
 
 _css = None
+
+
 def css():
     global _css
     if _css is None:
@@ -36,6 +38,16 @@ def css():
     return _css
 
 
+def copy_all(web_view):
+    web_view = getattr(web_view, 'details', web_view)
+    mf = web_view.page().mainFrame()
+    c = QApplication.clipboard()
+    md = QMimeData()
+    md.setText(mf.toPlainText())
+    md.setHtml(mf.toHtml())
+    c.setMimeData(md)
+
+
 def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=None):  # {{{
     table, comment_fields = (render_data_func or render_data)(mi, all_fields=all_fields,
             use_roman_numbers=config['use_roman_numerals_for_series_number'])
@@ -89,6 +101,7 @@ def render_html(mi, css, vertical, widget, all_fields=False, render_data_func=No
                 % (table, right_pane))
     return ans
 
+
 def get_field_list(fm, use_defaults=False):
     from calibre.gui2.ui import get_gui
     db = get_gui().current_db
@@ -109,6 +122,7 @@ def get_field_list(fm, use_defaults=False):
     available = frozenset(fm.displayable_field_keys())
     return [(f, d) for f, d in fieldlist if f in available]
 
+
 def render_data(mi, use_roman_numbers=True, all_fields=False):
     field_list = get_field_list(getattr(mi, 'field_metadata', field_metadata))
     field_list = [(x, all_fields or display) for x, display in field_list]
@@ -117,6 +131,7 @@ def render_data(mi, use_roman_numbers=True, all_fields=False):
 
 # }}}
 
+
 def details_context_menu_event(view, ev, book_info):  # {{{
     p = view.page()
     mf = p.mainFrame()
@@ -127,6 +142,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
     for action in list(menu.actions()):
         if action is not ca:
             menu.removeAction(action)
+    menu.addAction(QIcon(I('edit-copy.png')), _('Copy &all'), partial(copy_all, book_info))
     if not r.isNull():
         if url.startswith('format:'):
             parts = url.split(':')
@@ -210,6 +226,7 @@ def details_context_menu_event(view, ev, book_info):  # {{{
         menu.exec_(ev.globalPos())
 # }}}
 
+
 class CoverView(QWidget):  # {{{
 
     cover_changed = pyqtSignal(object, object)
@@ -404,7 +421,7 @@ class CoverView(QWidget):  # {{{
         except:
             sz = QSize(0, 0)
         self.setToolTip(
-            '<p>'+_('Double-click to open Book Details window') +
+            '<p>'+_('Double-click to open the Book details window') +
             '<br><br>' + _('Path') + ': ' + current_path +
             '<br><br>' + _('Cover size: %(width)d x %(height)d')%dict(
                 width=sz.width(), height=sz.height())
@@ -414,6 +431,7 @@ class CoverView(QWidget):  # {{{
 
 # Book Info {{{
 
+
 class BookInfo(QWebView):
 
     link_clicked = pyqtSignal(object)
@@ -619,6 +637,7 @@ class DetailsLayout(QLayout):  # {{{
 
 # }}}
 
+
 class BookDetails(QWidget):  # {{{
 
     show_book_info = pyqtSignal()
