@@ -41,6 +41,7 @@ entry_points = {
         ],
       }
 
+
 class PreserveMIMEDefaults(object):
 
     def __init__(self):
@@ -86,6 +87,8 @@ class PreserveMIMEDefaults(object):
                         raise
 
 # Uninstall script {{{
+
+
 UNINSTALL = '''\
 #!{python}
 from __future__ import print_function, unicode_literals
@@ -155,7 +158,7 @@ for f in mr:
 
 print ()
 
-if mimetype_icons and raw_input('Remove the ebook format icons? [y/n]:').lower() in ['', 'y']:
+if mimetype_icons and raw_input('Remove the e-book format icons? [y/n]:').lower() in ['', 'y']:
     for i, (name, size) in enumerate(mimetype_icons):
         remove_icon('mimetypes', name, size, update=i == len(mimetype_icons) - 1)
 '''
@@ -164,6 +167,7 @@ if mimetype_icons and raw_input('Remove the ebook format icons? [y/n]:').lower()
 
 # Completion {{{
 
+
 class ZshCompleter(object):  # {{{
 
     def __init__(self, opts):
@@ -286,6 +290,7 @@ class ZshCompleter(object):  # {{{
         w('\n}\n')
 
         log = DevNull()
+
         def get_parser(input_fmt='epub', output_fmt=None):
             of = ('dummy2.'+output_fmt) if output_fmt else 'dummy'
             return create_option_parser(('ec', 'dummy1.'+input_fmt, of, '-h'), log)[0]
@@ -484,6 +489,7 @@ _ebook_edit() {
                 f.write('esac\n')
 # }}}
 
+
 def get_bash_completion_path(root, share, info):
     if root == '/usr':
         # Try to get the system bash completion dir since we are installing to
@@ -499,6 +505,7 @@ def get_bash_completion_path(root, share, info):
         # Use the default bash-completion dir under staging_share
         return os.path.join(share, 'bash-completion', 'completions', 'calibre')
 
+
 def write_completion(bash_comp_dest, zsh):
     from calibre.ebooks.metadata.cli import option_parser as metaop, filetypes as meta_filetypes
     from calibre.ebooks.lrf.lrfparser import option_parser as lrf2lrsop
@@ -528,6 +535,7 @@ def write_completion(bash_comp_dest, zsh):
         def o_and_e(*args, **kwargs):
             f.write(opts_and_exts(*args, **kwargs))
             zsh.opts_and_exts(*args, **kwargs)
+
         def o_and_w(*args, **kwargs):
             f.write(opts_and_words(*args, **kwargs))
             zsh.opts_and_words(*args, **kwargs)
@@ -553,7 +561,7 @@ def write_completion(bash_comp_dest, zsh):
             '--exec-file':['py', 'recipe'],
             '--add-simple-plugin':['py'],
             '--inspect-mobi':['mobi', 'azw', 'azw3'],
-            '--viewer':list(available_input_formats()),
+            '--viewer':sorted(available_input_formats()),
         })
         f.write(textwrap.dedent('''
         _ebook_device_ls()
@@ -632,6 +640,7 @@ def write_completion(bash_comp_dest, zsh):
     zsh.write()
 # }}}
 
+
 class PostInstall:
 
     def task_failed(self, msg):
@@ -853,6 +862,7 @@ class PostInstall:
 
     # }}}
 
+
 def option_parser():
     from calibre.utils.config import OptionParser
     parser = OptionParser()
@@ -879,6 +889,7 @@ def options(option_parser):
         opts.extend(opt._long_opts)
     return opts
 
+
 def opts_and_words(name, op, words, takes_files=False):
     opts  = '|'.join(options(op))
     words = '|'.join([w.replace("'", "\\'") for w in words])
@@ -909,8 +920,10 @@ def opts_and_words(name, op, words, takes_files=False):
 }
 complete -F _'''%(opts, words) + fname + ' ' + name +"\n\n").encode('utf-8')
 
+
 pics = {'jpg', 'jpeg', 'gif', 'png', 'bmp'}
 
+
 def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
                   file_map={}):
     opts = ' '.join(options(op))
@@ -927,7 +940,7 @@ def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
     extras = []
     for eopts, eexts in ((cover_opts, "${pics}"), (opf_opts, "'@(opf)'")):
         for opt in eopts:
-            extras.append(special_exts_template%(opt, eexts))
+            extras.append(special_exts_template%(opt, sorted(eexts)))
     extras = '\n'.join(extras)
 
     return '_'+fname+'()'+\
@@ -958,7 +971,7 @@ def opts_and_exts(name, op, exts, cover_opts=('--cover',), opf_opts=(),
 
 }
 complete -o filenames -F _'''%dict(pics=spics,
-    opts=opts, extras=extras, exts=exts) + fname + ' ' + name +"\n\n"
+    opts=opts, extras=extras, exts=sorted(exts)) + fname + ' ' + name +"\n\n"
 
 
 VIEWER = '''\
@@ -1014,6 +1027,7 @@ Icon=calibre-gui
 Categories=Office;
 '''
 
+
 def get_appdata():
     _ = lambda x: x  # Make sure the text below is not translated, but is marked for translation
     return {
@@ -1022,7 +1036,7 @@ def get_appdata():
             'summary':_('The one stop solution to all your e-book needs'),
             'description':(
                 _('calibre is the one stop solution to all your e-book needs.'),
-                _('You can use calibre to catalog your books, fetch metadata for them automatically, convert them from and to all the various ebook formats, send them to your e-book reader devices, read the books on your computer, edit the books in a dedicated e-book editor and even make them available over the network with the built-in content server. You can also download news and periodicals in e-book format from over a thousand different news and magazine websites.')  # noqa
+                _('You can use calibre to catalog your books, fetch metadata for them automatically, convert them from and to all the various e-book formats, send them to your e-book reader devices, read the books on your computer, edit the books in a dedicated e-book editor and even make them available over the network with the built-in Content server. You can also download news and periodicals in e-book format from over a thousand different news and magazine websites.')  # noqa
             ),
             'screenshots':(
                 (1408, 792, 'https://lh4.googleusercontent.com/-bNE1hc_3pIc/UvHLwKPGBPI/AAAAAAAAASA/8oavs_c6xoU/w1408-h792-no/main-default.png',),
@@ -1059,6 +1073,7 @@ def get_appdata():
         },
     }
 
+
 def write_appdata(key, entry, base, translators):
     from lxml.etree import tostring
     from lxml.builder import E
@@ -1102,22 +1117,23 @@ def render_img(image, dest, width=128, height=128):
     img = QImage(I(image)).scaled(width, height, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
     img.save(dest)
 
+
 def main():
     p = option_parser()
     opts, args = p.parse_args()
     PostInstall(opts)
     return 0
 
+
 def cli_index_strings():
     return _('Command Line Interface'), _(
         'On OS X, the command line tools are inside the calibre bundle, for example,'
     ' if you installed calibre in :file:`/Applications` the command line tools'
     ' are in :file:`/Applications/calibre.app/Contents/console.app/Contents/MacOS/`.'), _(
-        'Documented Commands'), _('Undocumented Commands'), _(
+        'Documented commands'), _('Undocumented commands'), _(
         'You can see usage for undocumented commands by executing them without arguments in a terminal.'), _(
-            'Change Language')
+            'Change language')
 
 
 if __name__ == '__main__':
     sys.exit(main())
-
